#!/usr/bin/perl -w
##############################################################
# gestionnaire de tches planifies
# perldoc plutar.pl pour les details
##############################################################
use strict;
use FDN::Common::Constantes;
use FDN::Common::Logger;
use FDN::Common::Modules;
use FDN::Common::Config;
use FDN::Sql::Plutar::Run;
use FDN::Sql::Plutar::Basic;
use FDN::Sql::Plutar::Constantes;
use Getopt::Long qw(:config no_ignore_case);
use Mail::Sendmail;

##############################################################
# variables globales
##############################################################
my $base	= undef;
my $verbose 	= FALSE;	# flag de verbosit
my $debug 	= FALSE;	# flag de debugage
my $conf	= undef;	# le chemin du fichier de conf  utiliser
my $help	= FALSE;	# doit on afficher l'aide et sortir 
my $cr		= undef;        # Le nouveau CONFIG_ROOT s'il y a lieu
my @modules     = ();

my $nb_tasks	= 0;		# le nombre de tches traites


##############################################################
# prototypes
##############################################################

sub exit_with_error($);	# sort du programme en affichant le message fourni
sub __verbose($);	# affiche un message si -v ou -d activ (et le log en tant qu'info)
sub __debug($);		# affiche un message si -d  activ (et le log en tant que debug)
sub help();		# affiche l'utilisation et sort

##############################################################
# rcupration des arguments
##############################################################
GetOptions (	"verbose" => \$verbose	,
		"debug"	  => \$debug	,
		"help"	  => \$help	,
		"conf=s"  => \$conf     ,
		"Module=s"=> \@modules  ,
		"Conf=s"  => \$cr
);
help() if $help;

if ( defined $cr ) {
	__verbose "Config root changed to $cr";
	$FDN::Common::Config::CONFIG_ROOT = $cr;
}

if ( defined $conf ) {
	my $tmp = plutar_select_config($conf);
	if ( ! $tmp ) {
		print STDERR "Cannot read the configuration file $conf\n";
		help();
	}
}

my $ok = TRUE;
for my $m ( @modules ) {
	if ( ! conduse($m) ) {
		print "Failed to load module $m\n";
		$ok = FALSE;
	}
}
if ( !$ok ) {
	print STDERR "At least one module failed. Giving up.\n";
	help();
}

$base = Config_Key($conf,'common','base');
my $dbh = plutar_select_db($base);
if ( ! defined $dbh ) {
	print STDERR "Failed to open database $base. See syslog files.\n";
	help();
}


##############################################################
# boucle de traitement sur les tches
##############################################################
my $task;
do {
	lock_plutar();
	$task = first_plutar();
	if ( ! defined $task ) {
		unlock_plutar();
		goto END;
	}
	my $pid = $task->{'PLUTAR_ID'};
	my $act = $task->{'PLUTAR_ACTION'};
	my $run = $task->{'PLUTAR_NBRUN'};
	plutar_runing_task($pid,$run+1);
	unlock_plutar();			# dvrouillage de la table PLUTAR
	
	# gestion de la tche choisie
	my $args = get_plutar_properties($pid);	# rcupration de ses arguments
	my $res  = plutar_run_task($args,$act);	# excution de la commande relative  la tche
	update_plutar_properties($pid,$args);
	
	# on modifie la tache en fonction du rsultat 	
	my $msg = plutar_after_run($pid,$res,$act);
	__verbose($msg);
	# si besoin, on envoie un mail de notice
	if ( plutar_send_notify($pid,$res,$act) ) {
		__verbose('A mail have been sent to notify the result of this run.');
	}
	$nb_tasks++;
END:
} while (defined $task);

__verbose ("aucune tche traite") if $nb_tasks == 0;
__verbose ("$nb_tasks tches traites") if $nb_tasks > 1;
__verbose ("$nb_tasks tche traite") if $nb_tasks == 1;

plutar_close_db();

##############################################################
# sous-routines
##############################################################

sub help() {
	print STDERR "PLUTAR - Programme Lger Unifiant les Traitements des Activits Retardes\n";
	print STDERR "utilisation de plutar :\n";
	print STDERR "plutar -c=<FICHIER> [-d|-v]\n";
	print STDERR "- FICHIER est un fichier de configuration plutar relativement a /etc/fdn/\n";
	print STDERR "- les options -d et -v permettent d'activer les flags de verbosit :\n";
	print STDERR "\t -d : debug, messages destins au dvelopeur\n";
	print STDERR "\t -v : verbose, messages destins  l'utilisateur\n";
	exit(1);
}

sub exit_with_error ($) {
	my $message = shift;
	error ($message);
	print "$message\n";
	exit(1);
}

sub __debug ($) {
	my $message = shift;
	return unless $message;
	return unless $debug;
	print $message."\n";
	debug $message;
}

sub __verbose ($) {
	my $message = shift;
	return unless $message;
	print $message."\n" if $verbose or $debug;
	info $message;
}

__END__
=pod

=head1 Plutar

Programme Lger Unifiant les Traitements des Activits Retardes
Process Lightweight Unifying Tasks like At with Retry

=head1 Type 

utilitaire systme

=head1 Role

Automate permettant le traitement de tche retardes de manire cohrente avec le monde des Sitalibs.
Il est possible de rentrer dans une base de donnes une liste de tches  excuter  divers moment,
cet utilitaire se chargera de les excutyer au bon moment, en gardant un suivi des rsultat, en rssayant 
s'il rencontre des checs...

=head1 Utilisation

plutar [-d|--debug, -v|--verbose] --file=<fichier>

--file (ou -f) est une option obligatoire, elle dsigne le fichier de configuration  prendre en 
compte pour excuter plutar. le chemin est relatif par rapport  la racine /etc/sitalibs (standard Sitalibs::Config)

--verbose (ou -v) est un flag qui activera l'affichage  l'cran d'infos destines  l'utilisateur (peu verbeux)

--debug (ou -d) est un flag qui activera l'affichage  l'cran d'infos destines au dveloppeur (trs verbeux)

=head1 Retours

????

=cut
