#!/usr/bin/perl

use strict;
use DateTime;
use Mail::Sendmail;
use FDN::Common::Constantes;
use FDN::Common::Config;
use FDN::Common::Logger;

sub init_all();
sub time_to_epoch($$$);
sub end_of_proc($);
sub __debug($);

my $real_mail = 0;
my $proc = {};
my $levels = {};
my @levels;
my $logdir;
my $year;
my $month;
my $tzone;
my $debug = 0;
my $Mois = {
	'Jan' => 1,
	'Feb' => 2,
	'Mar' => 3,
	'Apr' => 4,
	'May' => 5,
	'Jun' => 6,
	'Jul' => 7,
	'Aug' => 8,
	'Sep' => 9,
	'Oct' => 10,
	'Nov' => 11,
	'Dec' => 12
};
my $from;
my $to;
my $level_mail = 0;
my $level_save = 0;
my $time_shift = 0;

init_all();
while (<STDIN>) {
	my $line = $_;
	if ( $line !~ /^(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+):\s+(\S.*)$/ ) {
		next;
	}
	my ( $mois, $jour, $heure, $machine, $process, $ligne ) =
		($1, $2, $3, $4, $5, $6);
#	print STDERR "Process: $process, mois <$mois> jour <$jour> heure <$heure>\n";
	
	if ( $process !~ /^(.*)(\[\d+\])$/ ) {
		error "Ligne qui ne vient pas de syslog\n";
		next;
	}
	my ( $appli, $pid ) = ( $1, $2 );
	$pid = "$pid-$appli";
	
	if ( $ligne !~ /^(\w+)\s+\*\s+(\S.*)$/ ) {
		__debug "Line is not from FDN::Common::Logger";
		next;
	}
	my ( $level, $msg ) = ( $1, $2 );
	my $niveau = $levels->{$level};
	if ( ! defined $niveau ) {
		error "Level '$level' is not defined, dropping line: $line";
		next;
	}
	
	my $date = time_to_epoch($mois,$jour,$heure);
	if ( ! defined $date ) {
		error "Failed to read the date (seems to be before epoch!), dropping the line: $line";
		next;
	}
	if ( $date < 0 ) {
		error "Failed to read the date (wrong format), dropping the line: $line";
		next;
	}
	
	if ( ! defined $proc->{$pid} ) {
		$proc->{$pid} = {
				"start" => $date,
				"last"  => $date,
				"level" => 0,
				"lines" => []
				};
	}
	if ( $proc->{$pid}->{"last"} + 20 < $date ) {
		end_of_proc($pid);
		$proc->{$pid} = {
				"start" => $date,
				"last"  => $date,
				"level" => 0,
				"lines" => []
				};
	}
	$proc->{$pid}->{"last"} = $date;
	push @{$proc->{$pid}->{"lines"}}, $line;
	if ( $level > $proc->{$pid}->{"level"} ) {
		$proc->{$pid}->{"level"} = $level;
	}
}

for my $pid ( keys %$proc ) {
	end_of_proc($pid);
}


sub init_all() {
	# Init date to check for time-wrap.
	my $dt = DateTime->now();
	$year  = $dt->year;
	$month = $dt->month;
	# Read config
	$from    = Config_Key('logs.cfg','mail','from');
	$to      = Config_Key('logs.cfg','mail','to');
	$logdir  = Config_Key('logs.cfg','common','logdir');
	$tzone   = Config_Key('logs.cfg','common','timezone');
	my $lvls = Config_Key('logs.cfg','common','@levels');
	my $l = 0;
	for my $lvl ( @$lvls ) {
		__debug "Registering level $lvl as number $l";
		$levels->{$lvl} = $l;
		$levels[$l] = $lvl;
		$l++;
	}
	$level_mail = Config_Key('logs.cfg','common','level-mail');
	if ( ! defined $level_mail ) {
		error "Level for mail not defined. Using 'error'.";
		$level_mail = 'error';
	}
	$level_mail = $levels->{$level_mail};
	if ( ! defined $level_mail ) {
		error "Level for mail invalid. Using 4.";
		$level_mail = 4;
	}
	$level_save = Config_Key('logs.cfg','common','level-save');
	if ( ! defined $level_save ) {
		error "Level for saving not defined. Using 'notice'.";
		$level_save = 'notice';
	}
	$level_save = $levels->{$level_save};
	if  ( ! defined $level_save ) {
		error "Level for saving invalid. Using 1.";
		$level_save = 1;
	}
	if ( $level_save > $level_mail ) {
		error "I should save at least as much as I send by email. ".
		      "Your configuration file states differently. I change ".
		      "that to be more coherent.";
		$level_save = $level_mail;
	}
	$real_mail = Config_Key('logs.cfg','common','real-mail');
	if ( ! defined $real_mail ) {
		$real_mail = 1;
	}
	$time_shift = Config_Key('logs.cfg','common','time-shift');
	if ( ! defined $time_shift ) {
		$time_shift = 20;
	}
	$debug = Config_Key('logs.cfg','common','debug');
	if ( ! defined $debug ) {
		$debug = 0;
	}
}

sub time_to_epoch($$$) {
	my ( $mois, $jour, $hms ) = @_;
	if ( $hms !~ /^(\d+):(\d+):(\d+)$/ ) {
		print STDERR "CA MATCH PAS!\n";
	}
	my ( $h, $m, $s ) = ( $1, $2, $3 );
	return -1 if ( ! defined $h );
	return -2 if ( ! defined $m );
	return -3 if ( ! defined $s );
	$mois = $Mois->{$mois};
	return -4 if ( ! defined $mois );
	my $an;
	if ( $mois > $month ) {
		$an = $year -1;
	} else {
		$an = $year;
	}
	my $dt = DateTime->new(
		year => $an,
		month => $mois,
		day => $jour,
		hour => $h,
		minute => $m,
		second => $s,
		time_zone => $tzone
		);
	return $dt->epoch;
}

sub end_of_proc($) {
	my ( $pid ) = @_;
	__debug "Le process $pid a un niveau ".$proc->{$pid}->{'level'};
	__debug "Il a produit ".(scalar @{$proc->{$pid}->{"lines"}})." lignes de log";
	my $start = DateTime->from_epoch(
			epoch => $proc->{$pid}->{'start'},
			time_zone => $tzone
			);
	my $end   = DateTime->from_epoch(
			epoch => $proc->{$pid}->{'last'},
			time_zone => $tzone
			);

	my $sm = $start->month;
	if ( $sm =~ /^\d$/ ) {
		$sm = '0'.$sm;
	}
	my $sd = $start->day;
	if ( $sd =~ /^\d$/ ) {
		$sd = '0'.$sd;
	}
	my $st = $start->hms('');
	
	my @dirs = ( $logdir, $start->year, $sm, $sd );
	my $file = join '/', @dirs;
	$file   .= '/'.$st.'-'.$pid.'.log';

	my $logs = join '',@{$proc->{$pid}->{"lines"}};
	if ( $proc->{$pid}->{'level'} >= $level_mail ) {
		__debug "Il faut envoyer un mail";
		my $mail = '';
		$mail .= "Le process en erreur est $pid.\n";
		$mail .= "Il a commence a tourner le ".
			 $start->ymd." a ".$start->hms."\n";
		$mail .= "Il s'est termine le ".
			 $end->ymd." a ".$end->hms."\n";
		$mail .= "Il a envoye au moins un message de niveau ".
			 $levels[$proc->{$pid}->{'level'}]."\n";
		$mail .= "Ce n'est peut-etre pas grave, mais il faut regarder\n";
		$mail .= "Les logs de ce process sont diponibles dans ".$file."\n\n\n";
		$mail .= "Voila l'integrale des logs qu'il a produit:\n";
		$mail .= "--\n";
		$mail .= $logs;
		$mail .= "--\n".
			 "Ce message a ete envoye automatiquement\n\n\n\n";
		my $ret = sendmail (
			'To' => $to,
			'From' => $from,
			'Message' => $mail,
			'Subject' => "Erreur detectee pour $pid"
			) if ( $real_mail );
		print "Envoie de ce message:\n$mail" if ( ! $real_mail );
	}
	if ( $proc->{$pid}->{'level'} >= $level_save ) {
		__debug "Il faut sauver";
		if ( ! check_dir(@dirs) ) {
			error "Failed to create directory for $file";
			goto EOP;
		}
		if ( ! open OUT, ">$file" ) {
			error "Failed to open $file: $!";
			goto EOP;
		}
		print OUT $logs;
		close OUT;
	}
EOP:
	delete $proc->{$pid};
}

sub check_dir(@) {
	my ( @dirs ) = @_;
	my $dir = '';
	for my $d ( @dirs ) {
		$dir = $dir.$d.'/';
		if ( ! -d $dir ) {
			if ( ! mkdir $dir ) {
				error "Failed to create $dir: $!";
				return FALSE;
			}
		}
	}
	return TRUE;
}

sub __debug($) {
	return if ! $debug;
	debug @_;
}
