#!/usr/bin/perl

use strict;
use FDN::Common::Constantes;
use FDN::Common::Config;
use FDN::Common::Modules;
use FDN::Sql::Workflow::Basic;
use FDN::Sql::Workflow::Run;
use Getopt::Long qw(:config no_ignore_case);

my $help;
my $debug;
my $conf;
my $db;
my $cr;
my @modules;

GetOptions(	"debug" => \$debug,
		"help" => \$help,
		"conf=s" => \$conf,
		"db=s" => \$db,
		"Module=s" => \@modules,
		"Conf=s" => \$cr );

sub help() {
	print STDERR 
		"$0 [-d] [-h] [--conf conf-file.cfg] [-M perl::module] --db database\n".
		"-d: increase debug verbosity\n".
		"-h: this help message\n".
		"-c: the name of the configuration file (default relative\n".
		"    to /etc/fdn). Default is /etc/fdn/wf.cfg\n".
		"-M: load module <perl::module>, e.g. to have the functions\n".
		"    described in the workflow\n".
		"--db: the name of the database in /etc/fdn/dbc.cfg\n";
	exit(1);
}
help() if ( $help );

if ( defined $cr ) {
	print STDERR "Config root to $cr\n";
	$FDN::Common::Config::CONFIG_ROOT = $cr;
}

if ( ! defined $db ) {
	print STDERR "You must give the database name.\n";
	help();
}

my $dbh = workflow_select_db($db);
if ( ! defined $dbh ) {
	print STDERR "Failed to open database $db. See syslog files.\n";
	help();
}

if ( defined $conf ) {
	my $tmp = workflow_select_config($conf);
	if ( ! $tmp ) {
		print STDERR "Failed to read $conf file.";
		help();
	}
}

my $ok = TRUE;
for my $m ( @modules ) {
	if ( ! conduse($m) ) {
		print STDERR "Failed to load module $m\n";
		$ok = FALSE;
	}
}
if ( ! $ok ) {
	print STDERR "At least one module failed. Giving up.\n";
	help();
}


my $run;
my $res;
lock_run();
$run = run_par_state('free');
while ( $run ) {
	print "Je traite le run ".$run->{'RUN_ID'}." il porte sur la tache ".
		$run->{'TASK_ID'}."\n";
	set_run_state($run->{'RUN_ID'},'busy');
	unlock_run();
	
	$res = run_task($run);
	if ( ! defined $res ) {
		print "Il y a eu un echec, consulter les logs.\n";
	}
	
	lock_run();
	$run = run_par_state('free');
}

workflow_close_db();

